@@ -1,8 +1,9 @@
 """srcomp command-line interface."""
 
+from __future__ import annotations
+
 import argparse
 import sys
-from typing import List, Optional
 
 from . import (
     add_delay,
@@ -19,6 +20,7 @@ from . import (
     schedule_league,
     scorer,
     shift_matches,
+    show_league_table,
     show_schedule,
     summary,
     top_match_points,
@@ -28,8 +30,8 @@ from . import (
 )
 
 
-def add_list_commands(subparsers: argparse._SubParsersAction) -> None:
-    def command(settings):
+def add_list_commands(subparsers: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
+    def command(settings: argparse.Namespace) -> None:
         commands = subparsers.choices.keys()
         print(" ".join(commands))
 
@@ -60,6 +62,7 @@ def argument_parser() -> argparse.ArgumentParser:
     schedule_league.add_subparser(subparsers)
     scorer.add_subparser(subparsers)
     shift_matches.add_subparser(subparsers)
+    show_league_table.add_subparser(subparsers)
     show_schedule.add_subparser(subparsers)
     summary.add_subparser(subparsers)
     top_match_points.add_subparser(subparsers)
@@ -70,7 +73,7 @@ def argument_parser() -> argparse.ArgumentParser:
     return parser
 
 
-def main(args: Optional[List[str]] = None) -> None:
+def main(args: list[str] | None = None) -> None:
     """Run as the CLI tool."""
     if args is None:
         args = sys.argv[1:]
