@@ -1,8 +1,17 @@
+from __future__ import annotations
+
+import argparse
+import io
+import sys
+import textwrap
+import warnings
 from contextlib import contextmanager
-from typing import Any, Iterator, Optional, Sequence, Type
+from typing import Any, cast, Iterable, Iterator, Sequence, TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from sr.comp.raw_compstate import RawCompstate
 
 API_TIMEOUT_SECONDS = 3
-SSH_TIMEOUT_SECONDS = 2
 DEPLOY_USER = 'srcomp'
 BOLD = '\033[1m'
 FAIL = '\033[91m'
@@ -10,16 +19,6 @@ OKBLUE = '\033[94m'
 ENDC = '\033[0m'
 
 
-def ssh_connection(host):
-    from paramiko import AutoAddPolicy, SSHClient
-
-    client = SSHClient()
-    client.load_system_host_keys()
-    client.set_missing_host_key_policy(AutoAddPolicy())
-    client.connect(host, username=DEPLOY_USER, timeout=SSH_TIMEOUT_SECONDS)
-    return client
-
-
 def format_fail(*args: object) -> str:
     msg = " ".join(map(str, args))
     return BOLD + FAIL + msg + ENDC
@@ -28,7 +27,7 @@ def format_fail(*args: object) -> str:
 @contextmanager
 def exit_on_exception(
     msg: str = '{0}',
-    kind: Type[Exception] = Exception,
+    kind: type[Exception] = Exception,
 ) -> Iterator[None]:
     try:
         yield
@@ -37,13 +36,52 @@ def exit_on_exception(
         exit(1)
 
 
+@contextmanager
+def guard_unicode_output(stream):
+    """
+    Cope with users environments not being able to handle unicode by softening
+    the display of characters they can't handle.
+
+    While I would ideally prefer not to have this sort of behaviour, as it can
+    happen mid-deploy (which we cannot roll back) it's far safer that we handle
+    these _somehow_ than error the deploy part-way through.
+
+    This is aimed at users running under certain Windows Subsystem for Linux
+    Operating Systems which default to a non utf-8 locale :(
+    """
+
+    encoding = stream.encoding
+
+    if encoding.lower() == 'utf-8':
+        # Happy path
+        yield
+        return
+
+    warnings.warn(
+        "Your locale does not support unicode. Some characters may not display correctly.",
+    )
+
+    orig_write = stream.write
+
+    def write(text, *a, **k):
+        text = text.encode(encoding, errors='backslashreplace').decode(encoding)
+        orig_write(text, *a, **k)
+
+    try:
+        stream.write = write
+        yield
+    finally:
+        stream.write = orig_write
+
+
 def print_fail(*args: object, **kargs: Any) -> None:
     print(format_fail(*args), **kargs)
 
 
-def print_buffer(buf):
-    prefix = '> '
-    print(prefix + prefix.join(buf.readlines()).strip())
+def print_buffer(buf: io.StringIO) -> None:
+    content = buf.getvalue().rstrip()
+    if content:
+        print(textwrap.indent(content, '> '))
 
 
 def get_input(prompt: str) -> str:
@@ -54,7 +92,7 @@ def get_input(prompt: str) -> str:
 def query(
     question: str,
     options: Sequence[str],
-    default: Optional[str] = None,
+    default: str | None = None,
 ) -> str:
     if default:
         assert default in options
@@ -77,10 +115,10 @@ def query(
             return default
 
 
-def query_bool(question: str, default_val: Optional[bool] = None) -> bool:
+def query_bool(question: str, default_val: bool | None = None) -> bool:
     options = ('y', 'n')
     if default_val is True:
-        default = 'y'  # type: Optional[str]
+        default: str | None = 'y'
     elif default_val is False:
         default = 'n'
     else:
@@ -97,11 +135,14 @@ def ref_compstate(host: str) -> str:
     return f'ssh://{DEPLOY_USER}@{host}/~/compstate.git'
 
 
-def deploy_to(compstate, host, revision, verbose):
+def deploy_to(compstate: RawCompstate, host: str, revision: str, verbose: bool) -> int:
     print(BOLD + f"Deploying to {host}:" + ENDC)
 
+    from fabric import Connection  # type: ignore[import]
+    from invoke.exceptions import UnexpectedExit  # type: ignore[import]
+
     # Make connection early to check if host is up.
-    with ssh_connection(host) as client:
+    with Connection(host, user=DEPLOY_USER) as connection:
         # Push the repo
         url = ref_compstate(host)
         # Make a new branch for this revision so that it's visible to
@@ -120,8 +161,13 @@ def deploy_to(compstate, host, revision, verbose):
             )
 
         cmd = f"./update '{revision}'"
-        _, stdout, stderr = client.exec_command(cmd)
-        retcode = stdout.channel.recv_exit_status()
+        stdout, stderr = io.StringIO(), io.StringIO()
+        try:
+            result = connection.run(cmd, out_stream=stdout, err_stream=stderr)
+        except UnexpectedExit as e:
+            result = e.result
+
+        retcode: int = result.exited
 
         if verbose or retcode != 0:
             print_buffer(stdout)
@@ -131,34 +177,35 @@ def deploy_to(compstate, host, revision, verbose):
         return retcode
 
 
-def get_deployments(compstate):
+def get_deployments(compstate: RawCompstate) -> list[str]:
     with exit_on_exception("Failed to get deployments from state ({0})."):
         return compstate.deployments
 
 
-def get_current_state(host):
-    from urllib.request import urlopen
-
-    import simplejson as json
+def get_current_state(host: str) -> str | None:
+    import requests
 
     url = f'http://{host}/comp-api/state'
     try:
-        page = urlopen(url, timeout=API_TIMEOUT_SECONDS)
-        raw_state = json.load(page)
+        response = requests.get(url, timeout=API_TIMEOUT_SECONDS)
+        response.raise_for_status()
+        raw_state = response.json()
     except Exception as e:
         print(e)
         return None
     else:
-        return raw_state['state']
+        # While this could be any JSON, the schema is that this is a string
+        # containing the commit hash of the compstate.
+        return cast(str, raw_state['state'])
 
 
-def check_host_state(compstate, host, revision, verbose):
+def check_host_state(compstate: RawCompstate, host: str, revision: str, verbose: bool) -> bool:
     """
     Compares the host state to the revision we want to deploy. If the
     host's state isn't in the history of the deploy revision then various
     options are presented to the user.
 
-    Returrns whether or not to skip deploying to the host.
+    Returns whether or not to skip deploying to the host.
     """
     SKIP = True
     UPDATE = False
@@ -209,7 +256,7 @@ def check_host_state(compstate, host, revision, verbose):
         return SKIP
 
 
-def require_no_changes(compstate):
+def require_no_changes(compstate: RawCompstate) -> None:
     if compstate.has_changes:
         print_fail(
             "Cannot deploy state with local changes.",
@@ -219,7 +266,7 @@ def require_no_changes(compstate):
         exit(1)
 
 
-def require_valid(compstate):
+def require_valid(compstate: RawCompstate) -> None:
     from sr.comp.validation import validate
 
     with exit_on_exception("State cannot be loaded: {0}"):
@@ -230,7 +277,11 @@ def require_valid(compstate):
         query_warn("State has validation errors (see above)")
 
 
-def run_deployments(args, compstate, hosts):
+def run_deployments(
+    args: argparse.Namespace,
+    compstate: RawCompstate,
+    hosts: Iterable[str],
+) -> None:
     revision = compstate.rev_parse('HEAD')
     for host in hosts:
         if not args.skip_host_check:
@@ -248,19 +299,20 @@ def run_deployments(args, compstate, hosts):
     print(BOLD + OKBLUE + "Done" + ENDC)
 
 
-def command(args):
+def command(args: argparse.Namespace) -> None:
     from sr.comp.raw_compstate import RawCompstate
 
-    compstate = RawCompstate(args.compstate, local_only=False)
-    hosts = get_deployments(compstate)
+    with guard_unicode_output(sys.stdout), guard_unicode_output(sys.stderr):
+        compstate = RawCompstate(args.compstate, local_only=False)
+        hosts = get_deployments(compstate)
 
-    require_no_changes(compstate)
-    require_valid(compstate)
+        require_no_changes(compstate)
+        require_valid(compstate)
 
-    run_deployments(args, compstate, hosts)
+        run_deployments(args, compstate, hosts)
 
 
-def add_options(parser):
+def add_options(parser: argparse.ArgumentParser) -> None:
     parser.add_argument('--verbose', action='store_true')
     parser.add_argument(
         '--skip-host-check',
@@ -269,7 +321,7 @@ def add_options(parser):
     )
 
 
-def add_subparser(subparsers):
+def add_subparser(subparsers: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
     help_msg = "Deploy a given competition state to all known hosts"
     parser = subparsers.add_parser(
         'deploy',
