@@ -1,7 +1,11 @@
+from __future__ import annotations
+
+import argparse
+
 __description__ = "List available MIDI output ports."
 
 
-def command(args):
+def command(args: argparse.Namespace) -> None:
     import mido  # type: ignore[import]
 
     ports = mido.get_output_names()
@@ -10,7 +14,7 @@ def command(args):
         print("-", port)
 
 
-def add_subparser(subparsers):
+def add_subparser(subparsers: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
     parser = subparsers.add_parser(
         'list-midi-ports', help=__description__,
         description=__description__,
