@@ -1,23 +1,30 @@
+from __future__ import annotations
+
+import argparse
+from pathlib import Path
+from typing import DefaultDict
+
+from sr.comp.types import GamePoints, MatchId, TLA
+
 __description__ = "Summaries the teams scoring the most match points"
 
 
-def command(settings):
-    import os.path
+def command(settings: argparse.Namespace) -> None:
     from collections import defaultdict
-    from functools import partial
     from itertools import chain
 
     from sr.comp.comp import SRComp
 
-    comp = SRComp(os.path.realpath(settings.compstate))
+    comp = SRComp(settings.compstate)
 
     all_scores = (comp.scores.tiebreaker, comp.scores.knockout, comp.scores.league)
     all_points = dict(chain.from_iterable(s.game_points.items() for s in all_scores))
 
-    points_map = defaultdict(partial(defaultdict, list))
+    points_map: DefaultDict[GamePoints, DefaultDict[TLA, list[MatchId]]]
+    points_map = defaultdict(lambda: defaultdict(list))
 
-    for match, points in all_points.items():
-        for tla, team_points in points.items():
+    for match, match_points in all_points.items():
+        for tla, team_points in match_points.items():
             points_map[team_points][tla].append(match)
 
     count = len(points_map)
@@ -28,14 +35,14 @@ def command(settings):
         else:
             print()
             print(f"The following {len(team_info)} team(s) scored {points} points:")
-            for tla, matches in team_info.items():
+            for tla, matches in sorted(team_info.items()):
                 print(f"- {tla} in match(es): " + ", ".join(
                     "{}{}".format(*x)
                     for x in matches
                 ))
 
 
-def add_subparser(subparsers):
+def add_subparser(subparsers: argparse._SubParsersAction[argparse.ArgumentParser]) -> None:
     parser = subparsers.add_parser(
         'top-match-points',
         help=__description__,
@@ -44,5 +51,6 @@ def add_subparser(subparsers):
     parser.add_argument(
         'compstate',
         help="competition state repo",
+        type=Path,
     )
     parser.set_defaults(func=command)
